	page 66, 132
	title Binary (bit) oriented STR and VAL for BASIC
	subttl By Jim Mack, Editing Services Co.

odg	equ	<offset dgroup>

comment |

	Implements routines to generate and interpret binary strings.

	Two procedures are declared in BASIC:

	DECLARE FUNCTION BitStr$ (word%)
	DECLARE FUNCTION BitVal& (ones$)  'or BitVal% (ones$)

	BitStr returns a 16-byte string in the form "0010010000111001"
	 with 1's corresponding to set bits in WORD%

	BitVal returns the value of a string such as the one above.  Only the
	 first 16 places in the string are evaluated.  Fewer may be used.
	 A null string returns a zero result.

	NOTE: BitVal can be declared as an integer function instead, since
	the entire value is returned in the lower 16 bits.  If you use it
	that way, returned vales are signed integers: a string whose highest
	bit is set will return a negative value.  As a long integer function,
	values are always positive.

	| comment ends

	page+
	.model medium, basic
									page+
	.data

; This is a string descriptor and associated string body for our use

gstr	dw	16		;this is a valid QB string descriptor...
	dw	xstr

xstr	db	16 dup (0)	;...which refers to this string

	.code

BitStr		PROC uses es di, vlu

	cld
	mov	ax, ds
	mov	es, ax			; this so "stosb" will work for sure
	assume	es:@data
	mov	bx, vlu
	mov	dx, [bx]		; dx = word containing bits to test
	mov	cx, gstr		; number of bit positions
	mov	di, odg:xstr		; point to beginning of our own string
	mov	ah, '0'
@@:	mov	al, ah			; for each bit, start with "0"
	shl	dx, 1			; shift a bit into carry
	adc	al, 0			; makes "0" into "1" if bit is set
	stosb				; put "0" or "1" into string
	loop	@b			; until all 16 bits tested
	mov	ax, odg:gstr		; pointer to our own descriptor
	ret

BitStr		ENDP
									page+
BitVal		PROC uses es di si, str

;  The value is taken in two steps: first, the passed string is right-
;  justified (RSET) into a local string accumulator of exactly 16 bytes,
;  with ASCII bias removed.  Then the local string is evaluated to a
;  numeric result.  This allows strings of fewer than 16 bytes to be
;  aligned correctly, and illegal values to be detected easily.

	cld
	mov	ax, ds
	mov	es, ax
	assume	es:@data
	xor	ax, ax			; clear result in case no string
	mov	bx, str
	mov	cx, [bx]		; length of passed string
	or	cx, cx
	jz	bv99			; null string passed
	mov	di, odg:xstr		; will end up pointing off the end
	mov	cx, 8
	rep	stosw			; zero our string accumulator
	mov	cx, [bx]
	cmp	cx, 16			; if < 16, use passed length
	jna	@f
	mov	cx, 16			; maximum string is 16 characters
@@:	mov	si, 2[bx]		; address of string data
	add	si, cx
	std				; work backwards...
	dec	si			; ...from ends of strings...
	dec	di			; ...to right-align the data
	xor	bx, bx			; clear numeric accumulator
@@:	lodsb				; get string byte
	cmp	al, '1'			; abort on illegal characters
	ja	bv90
	cmp	al, '0'
	jb	bv90
	and	al, 1			; remove ASCII bias
	stosb				; put into string accumulator
	loop	@b
	cld				; string accumulator now full
	mov	cx, 16
	mov	si, odg:xstr		; turn it into numeric equivalent
@@:	lodsb
	xor	ah, ah
	sub	ah, al			; set carry for rotate if '1'
	rcl	bx, 1			; set bit in result
	loop	@b			; repeat until string empty
bv90:
	mov	ax, bx			; accumulator to result
bv99:
	xor	dx, dx			; make valid for INT or LONG
	cld				; clean up flags or die later
	ret

BitVal		ENDP

	END
