/*******************************************************************************
**                                                                            **
**                      Kermit for Microsoft Windows                          **
**                      ----------------------------                          **
**                               KERMDEB.C                                    **
**                                                                            **
**  This module contains the debugging routines.                              **
**                                                                            **
*******************************************************************************/

// INCLUDES -------------------------------------------------------------------

#include "kermit.h"

//-----------------------------------------------------------------------------
VOID WriteDebug(int nLevel, PSTR pszFmt, ...)

//  Description of what function does.

//  Param(s):   x...............Description.

//  Returns:    Result description.

{
    char        sWork[128];
    va_list     pArg;
    int         nLen;

    if (nLevel > DebugLevel)
        return;

    va_start(pArg, pszFmt);
    nLen = vsprintf(sWork, pszFmt, pArg);
    va_end(pArg);
    sWork[nLen++] = '\n';
    sWork[nLen] = '\0';

    if (DebugStream == NULL)
        MessageBeep(0);
    if (DebugStream == stdout) {
        if (hAppWnd)
            MessageBox(hAppWnd, sWork, "Kermit", MB_OK | MB_ICONASTERISK);
        else
            MessageBox(GetActiveWindow(), sWork, "Kermit",
                       MB_OK | MB_ICONASTERISK);
    }
    else {
        fputs(sWork, DebugStream);
        fflush(DebugStream);
    }
}

//-----------------------------------------------------------------------------
VOID DebugInit(HANDLE hInstance, HANDLE hPrevInstance,
               LPSTR lpszCmdLine, int cmdShow)

//  Description of what function does.

//  Param(s):   x...............Description.

//  Returns:    Result description.

{
    char pszFileName[80];

    DebugLevel = GetProfileInt("Kermit", "DebLevel", 0);

    GetProfileString("Kermit", "DebDest", "", pszFileName,
                     sizeof(pszFileName));
    if (strcmp(pszFileName, "POPUP") == 0)
        DebugStream = stdout;
    else
        DebugStream = fopen(pszFileName, "w");

    WriteDebug(4, "Kermit starting...");
    WriteDebug(4, "hInstance=%u\nhPrevInstance=%u\n"
                  "lpszCmdLine=%Fs\ncmdShow=%i",
               hInstance, hPrevInstance, lpszCmdLine, cmdShow);
}

//-----------------------------------------------------------------------------
VOID DebugTerm()

//  Description of what function does.

//  Param(s):   x...............Description.

//  Returns:    Result description.

{
    WriteDebug(4, "Kermit ending...");

    if (DebugStream != stdout)
        fclose(DebugStream);
}

//-----------------------------------------------------------------------------
VOID DebAssert(char *filename, int linenum, int exp, char *expstr)

//  Description of what function does.

//  Param(s):   x...............Description.

//  Returns:    Result description.

{
    if (exp)
        return;

    WriteDebug(1, "ASSERTION FAILED - File: %s, Line: %d\n%s",
               filename, linenum, expstr);
}

//-----------------------------------------------------------------------------
VOID DebTrace(char *filename, int linenum, int nLevel, PSTR pszFmt, ...)

//  Description of what function does.

//  Param(s):   x...............Description.

//  Returns:    Result description.

{
    char        sWork[128];
    va_list     pArg;
    int         nLen;

    va_start(pArg, pszFmt);
    nLen = vsprintf(sWork, pszFmt, pArg);
    va_end(pArg);

    WriteDebug(nLevel, "TRACE - File: %s, Line: %d\n%s",
               filename, linenum, sWork);
}
