'Ŀ
'                                                                        
'                          Q B D I S K . B A S                           
'                                                                        
'                   Supplementary Source Code for the                    
'       The QBSCR Screen Routines for QuickBASIC 4.0+ Programmers        
'                              Version 2.0                               
'                                                                        
'                   (C) Copyright 1992 by Tony Martin                    
'                                                                        
'Ĵ
'                                                                        
'  This source code is copyright 1992 by Tony Martin.  You may change    
'  it to suit your programming needs, but you may not distribute any     
'  modified copies of the library itself.  I retain all rights to the    
'  source code and all library modules included with the QBSCR package,  
'  as well as to the example programs.  You may not remove this notice   
'  from any copies of the library itself you distribute.                 
'                                                                        
'  You are granted the right to use this source code for your own pro-   
'  grams, without royalty payments or credits to me (though, if you      
'  feel so inclined to give me credit, feel free to do so).  You MUST    
'  register this software if you release a shareware or commercial       
'  program that uses it.  You may use these routines in any type of      
'  software you create, as long as it is not a programming toolbox or    
'  package of routines OF ANY KIND.                                      
'                                                                        
'  This package is shareware.  If you find it useful or use it in any    
'  software you release, you are requested to send a registration fee of 
'  $25.00 (U.S. funds only) to:                                          
'                                                                        
'                            Tony Martin                                 
'                       1611 Harvest Green Ct.                           
'                          Reston, VA 22094                              
'                                                                        
'  All registered users receive an 'official' disk set containing the    
'  latest verison of the QBSCR routines.  For more information, see      
'  the QBSCR documentation.                                              
'                                                                        
'Ĵ
'                                                                        
'  For information on using these routines and incorporating them into   
'  your own programs, see the accompanying documentation.                
'                                                                        
'

'$INCLUDE: 'qbscr.inc'
'$INCLUDE: 'qb.bi'

CONST FALSE = 0, TRUE = NOT FALSE

COMMON SHARED mouseExists%, mouseState%

SUB FileInfo (fd AS FileInfoType, dta$)

	' 
	' This function extracts file information from the 43-byte DTA that is
	' filled in by a call to FirstFile% or NextFile%.  It expects a data
	' object of type FileInfoType to be passed in, which it will fill with
	' the appropriate data.
	' 

	' 
	' Determine a numeric attribute value, as well as a string that represents
	' all applicable attributes for the file.  There will be character in the
	' string for each attribute that applies.  The characters have the follow-
	' ing meanings:
	'
	'               Character   Attribute
	'               ---------   -----------------
	'                   R       Read Only
	'                   H       Hidden file
	'                   S       System file
	'                   V       Volume Label
	'                   D       Subdirectory
	'                   A       Archive
	' 
	fd.fattribute = ASC(MID$(dta$, 22, 1))
	IF fd.fattribute AND ReadOnlyAttr THEN
		attrstr$ = attrstr$ + "R"
	END IF
	IF fd.fattribute AND HiddenAttr THEN
		attrstr$ = attrstr$ + "H"
	END IF
	IF fd.fattribute AND SystemAttr THEN
		attrstr$ = attrstr$ + "S"
	END IF
	IF fd.fattribute AND VolLabelAttr THEN
		attrstr$ = attrstr$ + "V"
	END IF
	IF fd.fattribute AND SubdirAttr THEN
		attrstr$ = attrstr$ + "D"
	END IF
	IF fd.fattribute AND ArchiveAttr THEN
		attrstr$ = attrstr$ + "A"
	END IF
	fd.fattrstr = attrstr$
 
	' 
	' Calculate the file's time of creation or last modification.  The time is
	' an unsigned integer, and we must improvise our own by adding 65536
	' to the value to compensate for the sign bit.  Build time string last.
	' 
	temptime& = CVI(MID$(dta$, 23, 2))
	IF temptime& < 0 THEN temptime& = temptime& + 65536  ' Adjust for unsigned.
	hour% = (temptime& \ 2048) AND 31     ' Have to mask out the hours,
	minute% = (temptime& \ 32) AND 63     ' minutes, and seconds.
	second% = temptime& AND 31
	fd.ftime = LTRIM$(STR$(hour%)) + ":" + LTRIM$(STR$(minute%)) + ":" + LTRIM$(STR$(second%))
 
	' 
	' Calculate date and build a date string.
	' 
	tempdate& = CVI(MID$(dta$, 25, 2))
	month% = (tempdate& \ 32) AND 15      ' Have to mask out the month, day,
	day% = tempdate& AND &H31             ' and year (since 1980).
	year% = ((tempdate& \ 512) AND 31) + 1980
	fd.fdate = LTRIM$(STR$(month%)) + "-" + LTRIM$(STR$(day%)) + "-" + LTRIM$(STR$(year%))

	' 
	' File size is a simple long integer.
	' 
	fd.fsize = CVL(MID$(dta$, 27, 4))
	
	' 
	' File name starts at byte 31 in the DTA, and is terminated by an ASCII 0
	' (the NULL character).  We can find the end by looking for the NULL.  If
	' we don't, and just use all 13 characters, we may end with some characters
	' from a previous filename.
	' 
	fd.fname = MID$(dta$, 31, INSTR(RIGHT$(dta$, 13), CHR$(0)) - 1)
 
END SUB

FUNCTION FirstFile% (fileName$, attributes%, dta$)

	' 
	' This function searches for the first files who's name matches the
	' passed-in fileName$.  Note that fileName$ may have wildcards * and ? in
	' them.  The DTA$ is filled by this function (actually by the call to
	' the DOS service) and is used in later calls to FindNext% and FileInfo%.
	' 

	' 
	' Define a register type for us to use.
	' 
	DIM regx AS RegTypeX
	dta$ = SPACE$(43)

	' 
	' When DOS performs disk activity, it uses the DTA (Disk Transfer Area)
	' portion of memory.  Usually this is a 128 byte buffer located at offset
	' 80h in the PSP (Program Segment Prefix). If this doesn't make sense,
	' don't worry.  All you need to know is that since, we will be using our
	' own DTA (to store info about the last file search), we must save the
	' current DTA address and restore it when we are done.
	' 
	regx.ax = &H2F00        ' Service 2F (Get DTA) in register AH, 00 in AL.
	INTERRUPTX &H21, regx, regx
	oldOffset% = regx.bx    ' Register BX contains offset of current DTA.
	oldSegment% = regx.es   ' Register ES contains segment of current DTA.

	' 
	' Now we must define our own DTA for DOS to use.
	' 
	regx.ax = &H1A00        ' Service 1A (Set DTA) in register AH, 00 in AL.
	regx.dx = SADD(dta$)    ' Offset of address of dta$.
	regx.ds = VARSEG(dta$)  ' Segment of address of dta$.
	INTERRUPTX &H21, regx, regx
 
	' 
	' Set up data before call to function and call the interrupt service.
	' 
	tempName$ = fileName$ + CHR$(0)    ' NULL-terminate the filename.
	regx.ax = &H4E00
	regx.cx = attributes%
	regx.dx = SADD(tempName$)
	regx.ds = VARSEG(tempName$)
	INTERRUPTX &H21, regx, regx

	' 
	' If the Carry flag is ste, there was an error, i.e., no match was found.
	' 
	IF (regx.flags AND 1) THEN    ' ERROR!
		FirstFile% = FALSE          ' Return FALSE - no match found.
	ELSE
		FirstFile% = TRUE           ' Yay!  A match!
	END IF

	' 
	' Now that we're done, we have to restore the DOS DTA space we saved at
	' the beginning of the routine.
	' 
	regx.ax = &H1A00        ' Service 1A in register AH, 00 in AL.
	regx.dx = oldOffset%    ' Offset of address of original DOS DTA.
	regx.ds = oldSegment%   ' Segment of address of original DOS DTA.
	INTERRUPTX &H21, regx, regx

END FUNCTION

FUNCTION GetDiskFreeSpace& (drive%)

	' 
	' This function returns the amount of free disk space (in bytes) of the
	' specified drive.  The drive code passed-in (drive%) has values like this:
	' Drive code: 0=default, 1=A, 2=B, 3=C, etc.  Note that this function
	' returns a LONG INTEGER value.
	' 
 
	DIM reg AS RegType

	' 
	' We want service 36h in register AH, and the drive code in register DX.
	' 
	reg.ax = &H3600
	reg.dx = drive%
	INTERRUPT &H21, reg, reg

	' 
	' Calculate the total butes free. The service 36h returns the disk sectors
	' per cluster in register AX, the total available clusters in register BX,
	' and the bytes per cluster in register CX.  Multiply all three together to
	' get the total free space in bytes. This must be a LONG integer multipli-
	' cation.
	' 
	sectorsPerCluster& = reg.ax
	availClusters& = reg.bx
	bytesPerSector& = reg.cx

	GetDiskFreeSpace& = sectorsPerCluster& * availClusters& * bytesPerSector&

END FUNCTION

FUNCTION GetDriveCode%

	' 
	' This function determines the current default disk drive and returns its
	' DOS numeric code.  The codes look like 0=A, 1=B, 2=C, etc.  You probably
	' won't be using it a whole lot, but it is used by the NumDrives% function.
	' 
 
	DIM reg AS RegType

	' 
	' We want DOS service 19h in register AH.
	' 
	reg.ax = &H1900
	INTERRUPT &H21, reg, reg

	' 
	' The drive code is returned in the lower half of AX, or AH.
	' 
	GetDriveCode% = reg.ax AND &HFF

END FUNCTION

FUNCTION GetDriveLetter$

	' 
	' This function returns a single letter, the letter of the current default
	' disk drive.
	' 

	DIM reg AS RegType

	' 
	' We want DOS service 19h in register AH.
	' 
	reg.ax = &H1900
	INTERRUPT &H21, reg, reg

	' 
	' The drive code is returned in the lower half of AX, or AH.  We can easily
	' calculate the letter that represents this drive by adding 65 to it.
	' 
	GetDriveLetter$ = CHR$((reg.ax AND &HFF) + 65)

END FUNCTION

FUNCTION GetPath$ (drive%)

	' 
	' This function determines the current DOS path.  Not the path specified
	' by the PATH command, but rather the current path the user is logged to.
	' Simplay pass in a drive code, where drive% is like 0=default, 1=A, 2=B,
	' 3=C, etc.
	' 

	' 
	' Must make sure that there are 65 bytes reserved for the path name.
	' 
	path$ = SPACE$(65)
	DIM regx AS RegTypeX

	' 
	' Place DOS service 47h in register AH, the drive code in register DX, the
	' offset of the address of the path buffer in SI, and the segment of the
	' address of the path buffer in DS.
	' 
	regx.ax = &H4700
	regx.dx = drive%
	regx.si = SADD(path$)
	regx.ds = VARSEG(path$)
	INTERRUPTX &H21, regx, regx

	' 
	' If the carry flag (first bit position in the flags byte) is set, then
	' there was an error, so return a NULL string ("").  Otherwise, return
	' the string filled in by the DOS service.
	' 
	IF (regx.flags AND 1) THEN
		path$ = ""
	END IF
 
	GetPath$ = path$

END FUNCTION

FUNCTION NextFile% (dta$)

	' 
	' This function searches for any files who's name matches the data in then
	' passed-in dta$.  Note that the DTA$ is filled by this function (actually
	' by the call to the DOS service) and is used in later calls to FindNext%
	' and FileInfo%.
	' 

	' 
	' Define a register type for us to use.
	' 
	DIM regx AS RegTypeX

	' 
	' When DOS performs disk activity, it uses the DTA (Disk Transfer Area)
	' portion of memory.  Usually this is a 128 byte buffer located at offset
	' 80h in the PSP (Program Segment Prefix). If this doesn't make sense,
	' don't worry.  All you need to know is that since, we will be using our
	' own DTA (to store info about the last file search), we must save the
	' current DTA address and restore it when we are done.
	' 
	regx.ax = &H2F00        ' Service 2F (Get DTA) in register AH, 00 in AL.
	INTERRUPTX &H21, regx, regx
	oldOffset% = regx.bx    ' Register BX contains offset of current DTA.
	oldSegment% = regx.es   ' Register ES contains segment of current DTA.

	' 
	' Now we must define our own DTA for DOS to use.
	' 
	regx.ax = &H1A00        ' Service 1A (Set DTA) in register AH, 00 in AL.
	regx.dx = SADD(dta$)    ' Offset of address of dta$.
	regx.ds = VARSEG(dta$)  ' Segment of address of dta$.
	INTERRUPTX &H21, regx, regx

	' 
	' Set up data before call to function and call the interrupt service.
	' 
	regx.ax = &H4F00
	INTERRUPTX &H21, regx, regx

	' 
	' If the Carry flag is ste, there was an error, i.e., no match was found.
	' 
	IF (regx.flags AND 1) THEN    ' ERROR!
		NextFile% = FALSE          ' Return FALSE - no match found.
	ELSE
		NextFile% = TRUE           ' Yay!  A match!
	END IF

	' 
	' Now that we're done, we have to restore the DOS DTA space we saved at
	' the beginning of the routine.
	' 
	regx.ax = &H1A00        ' Service 1A in register AH, 00 in AL.
	regx.dx = oldOffset%    ' Offset of address of original DOS DTA.
	regx.ds = oldSegment%   ' Segment of address of original DOS DTA.
	INTERRUPTX &H21, regx, regx

END FUNCTION

FUNCTION NumDrives%

	DIM reg AS RegType

	reg.ax = &HE00
	reg.dx = GetDriveCode%    ' Find out current drive, so we stay there.
	INTERRUPT &H21, reg, reg

	NumDrives% = reg.ax AND &HFF

END FUNCTION

SUB SetDefaultDrive (drive%)
 
	' 
	' This routine allows you to set a new default drive.  The drive code that
	' is passed in looks like 0=A, 1=B, 2=C, etc.
	' 

	DIM reg AS RegType

	' 
	' DOS service 0Eh in register AH, and the drive code in register DX.
	' It's that simple!
	' 
	reg.ax = &HE00
	reg.dx = drive%
	INTERRUPT &H21, reg, reg

END SUB

