'*  QBFCOMMS.BAS
'*---------------------------------------------------------------------------
'*
'*  QuickBASIC Comms Routines using the X00 QB HLL module.
'*
'*  by Tom Collins
'*  Version 1.0
'*  10-15-90
'*

' $INCLUDE: 'QBFCOMMS.BI'

'*  FossilAnsiPrint()
'*---------------------------------------------------------------------------
'*
'*  Prints a string to the screen using ANSI processing.
'*
      FUNCTION FossilAnsiPrint% (A$)

      DIM InRegs AS RegType, OutRegs AS RegType

      FOR I = 1 TO LEN(A$)
	 B$ = MID$(A$, I, 1)
	 InRegs.ax = &H1300 + ASC(B$)    ' FOSSIL Service 13h
	 CALL QBX00(InRegs, OutRegs)
      NEXT

      FossilAnsiPrint% = 0

      END FUNCTION

'*  FossilBIOSPrint()
'*---------------------------------------------------------------------------
'*
'*  Prints a string to the screen using BIOS I/O.
'*
      FUNCTION FossilBIOSPrint% (A$)

      DIM InRegs AS RegType, OutRegs AS RegType

      FOR I = 1 TO LEN(A$)
	 B$ = MID$(A$, I, 1)
	 InRegs.ax = &H1500 + ASC(B$)    ' FOSSIL Service 15h
	 CALL QBX00(InRegs, OutRegs)
      NEXT

      FossilBIOSPrint% = 0

      END FUNCTION

'*  FossilBreak()
'*---------------------------------------------------------------------------
'*
'*  Starts (Ctrl = 1) or stops (Ctrl = 0) sending a BREAK signal
'*  to the modem.
'*
      FUNCTION FossilBreak% (Port%, Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H1A00 + Ctrl%         ' FOSSIL Service 1Ah

      CALL QBX00(InRegs, OutRegs)

      FossilBreak% = 0

      END FUNCTION

'*  FossilCtrlKCtrlC()
'*---------------------------------------------------------------------------
'*
'*  Enables or disables extended ^K and ^C FOSSIL Checking and returns
'*  with 1 (^C or ^K has been received) or 0 (hasn't been received)
'*
'*  See Masks above for Ctrl% values.
'*
      FUNCTION FossilCtrlKCtrlC% (Port%, Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H1000                 ' FOSSIL Service 10h

      CALL QBX00(InRegs, OutRegs)

      FossilCtrlKCtrlC% = OutRegs.ax

      END FUNCTION

'*  FossilDeactivate()
'*---------------------------------------------------------------------------
'*
'*  Deactivates the FOSSIL for a given port.
'*
      FUNCTION FossilDeactivate% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H500                  ' FOSSIL Service 05h

      CALL QBX00(InRegs, OutRegs)

      FossilDeactivate% = 0

      END FUNCTION

'*  FossilDTR()
'*---------------------------------------------------------------------------
'*
'*  Sets (Ctrl = 1) or Resets (Ctrl = 0) DTR for a given port and
'*  returns with the port status.
'*
      FUNCTION FossilDTR% (Port%, Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H600 + Ctrl%          ' FOSSIL Service 06h

      CALL QBX00(InRegs, OutRegs)

      FossilDTR% = 0

      END FUNCTION

'*  FossilFlowControl()
'*---------------------------------------------------------------------------
'*
'*  Enables or disables flow control for a given port.
'*
      FUNCTION FossilFlowControl% (Port%, Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &HF00 + Ctrl%          ' FOSSIL Service 0Fh

      CALL QBX00(InRegs, OutRegs)

      FossilFlowControl% = 0

      END FUNCTION

'*  FossilFlush()
'*---------------------------------------------------------------------------
'*
'*  Flushes the output buffer for a given port. (Waits for output
'*  to finish)
'*
      FUNCTION FossilFlush% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H800                  ' FOSSIL Service 08h

      CALL QBX00(InRegs, OutRegs)

      FossilFlush% = 0

      END FUNCTION

'*  FossilGetb()
'*---------------------------------------------------------------------------
'*
'*  Gets a buffer of charactes from a given port and returns the number
'*  of characters actually received.
'*
'*  Note: The string passed MUST!! have space available for the number
'*        of characters requested.
'*
'*  Usage: A$ = SPACE$(256)
'*         X = FossilGetb(COM1 ,A$, 256)
'*
      FUNCTION FossilGetb% (Port%, Buffer$, Length%)

      DIM InRegsX AS RegTypeX, OutRegsX AS RegTypeX

      InRegsX.dx = Port%
      InRegsX.ax = &H1800                ' FOSSIL Service 18h
      InRegsX.cx = Length%               ' CX = Characters to get
      InRegsX.es = VARSEG(Buffer$)       ' ES = Segment of Buffer$
      InRegsX.di = SADD(Buffer$)         ' DI = Offset of Buffer$

      CALL QBX00X(InRegsX, OutRegsX)

      FossilGetb% = OutRegsX.ax

      END FUNCTION

'*  FossilGetch()
'*---------------------------------------------------------------------------
'*
'*  Gets a character from a given port.
'*
'*  Note: You must check for data available before calling this
'*        functions, as it will wait forever for a given byte.
'*
      FUNCTION FossilGetch$ (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H200                  ' FOSSIL Service 02h

      CALL QBX00(InRegs, OutRegs)

      FossilGetch$ = CHR$(OutRegs.ax)

      END FUNCTION

'*  FossilInfo()
'*---------------------------------------------------------------------------
'*
'*  Gets a bunch of information about the FOSSIL.
'*
'*    Rev%  - FOSSIL revision this driver follows
'*    Ver%  - FOSSIL version this driver follows
'*    ID$   - ID string ("BNU Fossil Driver")
'*    ISZ%  - Input buffer size
'*    OSZ%  - Output buffer size
'*    IFR%  - Input buffer bytes free
'*    OFR%  - Output buffer bytes free
'*    SW%   - Screen width
'*    SH%   - Screen height
'*    Baud& - FOSSIL current baud rate (300,1200...)
'*
      FUNCTION FossilInfo% (Port%, Rev%, Ver%, ID$, ISZ%, IFR%, OSZ%, OFR%, SW%, SH%, Baud&)

      DIM InRegsX AS RegTypeX, OutRegsX AS RegTypeX
      DIM Info AS FossilInfoType

      InRegsX.dx = Port%
      InRegsX.ax = &H1B00                 ' FOSSIL Service 1Bh
      InRegsX.cx = 19                     ' Size of buffer to get
      InRegsX.es = VARSEG(Info)           ' Segment of buffer
      InRegsX.di = VARPTR(Info)           ' Offset of buffer

      CALL QBX00X(InRegsX, OutRegsX)

      Ver% = ASC(Info.MajorVersion)
      Rev% = ASC(Info.MinorVersion)

      ID$ = ""
      DEF SEG = Info.IdentSegment
      FOR I = 0 TO 100
	 C = PEEK(I + Info.IdentOffset)
	 IF C = 0 THEN
	    EXIT FOR
	 ELSE
	    ID$ = ID$ + CHR$(C)
	 END IF
      NEXT

      ISZ% = Info.InBufrSize
      OSZ% = Info.OutBufrSize
      IFR% = Info.InBufrFree
      OFR% = Info.OutBufrFree
      SW% = ASC(Info.ScreenWidth)
      SH% = ASC(Info.ScreenHeight)

      SELECT CASE FIX(ASC(Info.Baud) / 32)
	 CASE 3
	    Baud& = 600
	 CASE 4
	    Baud& = 1200
	 CASE 5
	    Baud& = 2400
	 CASE 6
	    Baud& = 4800
	 CASE 7
	    Baud& = 9600
	 CASE 0
	    Baud& = 19200
	 CASE 1
	    Baud& = 38400
	 CASE ELSE
	    Baud& = 300
      END SELECT

      FossilInfo% = 0

      END FUNCTION

'*  FossilInit()
'*---------------------------------------------------------------------------
'*
'*  Initializes the FOSSIL for a given port and returns 0 if successful,
'*  or 1 if not.
'*
      FUNCTION FossilInit% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H400                   ' FOSSIL Service 04h
      InRegs.flags = 512

      CALL QBX00(InRegs, OutRegs)

      IF OutRegs.ax = &H1954 THEN
	 FossilInit% = 0                  ' Success
      ELSE
	 FossilInit% = 1                  ' Fail
      END IF

      END FUNCTION

'*  FossilInkey()
'*---------------------------------------------------------------------------
'*
'*  Reads the next character in the keyboard buffer and returns it as a
'*  scan code, or -1 for none available.
'*
      FUNCTION FossilInkey%

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.ax = &HD00                  ' FOSSIL Service 0Dh

      CALL QBX00(InRegs, OutRegs)

      FossilInkey% = OutRegs.ax

      END FUNCTION

'*  FossilInkeyWait()
'*---------------------------------------------------------------------------
'*
'*  Reads the next character in the keyboard buffer and returns it as a
'*  scan code.  This function will wait forever for a key to be pressed.
'*
      FUNCTION FossilInkeyWait%

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.ax = &HE00                  ' FOSSIL Service 0Eh

      CALL QBX00(InRegs, OutRegs)

      FossilInkeyWait% = OutRegs.ax

      END FUNCTION

'*  FossilLocate()
'*---------------------------------------------------------------------------
'*
'*  Sets the cursor position to the row and column passed. (0,0) is
'*  the upper left corner.
'*
      FUNCTION FossilLocate% (Row%, Column%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.ax = &H1100                 ' FOSSIL Service 11h
      InRegs.dx = Row% * 256 + Column%

      CALL QBX00(InRegs, OutRegs)

      FossilLocate% = 0

      END FUNCTION

'*  FossilPeek()
'*---------------------------------------------------------------------------
'*
'*  Returns the next character in the input buffer for a given port
'*  but doesn't remove it from the buffer.  Returns "" if no data
'*  is available.
'*
      FUNCTION FossilPeek$ (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &HC00                  ' FOSSIL Service 0Ch

      CALL QBX00(InRegs, OutRegs)

      FossilPeek$ = ""
      IF OutRegs.ax <> -1 THEN
	 FossilPeek$ = CHR$(OutRegs.ax)
      END IF

      END FUNCTION

'*  FossilPurgeInputBuf()
'*---------------------------------------------------------------------------
'*
'*  Purges (throws away) the data in the input buffer for a given port.
'*
      FUNCTION FossilPurgeInputBuf% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &HA00                  ' FOSSIL Service 0Ah

      CALL QBX00(InRegs, OutRegs)

      FossilPurgeInputBuf% = 0

      END FUNCTION

'*  FossilPurgeOutputBuf()
'*---------------------------------------------------------------------------
'*
'*  Purges (throws away) the data in the output buffer for a given port.
'*
      FUNCTION FossilPurgeOutputBuf% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H900                  ' FOSSIL Service 09h

      CALL QBX00(InRegs, OutRegs)

      FossilPurgeOutputBuf% = 0

      END FUNCTION

'*  FossilPutb()
'*---------------------------------------------------------------------------
'*
'*  Writes a block of data to a given port and returns with the
'*  number of characters actually written.
'*
      FUNCTION FossilPutb% (Port%, Buffer$)

      DIM InRegsX AS RegTypeX, OutRegsX AS RegTypeX

      InRegsX.dx = Port%
      InRegsX.ax = &H1900                ' FOSSIL Service 19h
      InRegsX.cx = LEN(Buffer$)          ' CX = Chars to transmit
      InRegsX.es = VARSEG(Buffer$)       ' ES = Segment of Buffer$
      InRegsX.di = SADD(Buffer$)         ' DI = Offset of Buffer$

      CALL QBX00X(InRegsX, OutRegsX)

      FossilPutb% = OutRegsX.ax

      END FUNCTION

'*  FossilPutch()
'*---------------------------------------------------------------------------
'*
'*  Sends a character to a given port and returns 1 for success, 0 if not.
'*
      FUNCTION FossilPutch% (Port%, Char$)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &HB00 + ASC(Char$)     ' FOSSIL Service 0Bh

      CALL QBX00(InRegs, OutRegs)

      FossilPutch% = OutRegs.ax

      END FUNCTION

'*  FossilPutchWait()
'*---------------------------------------------------------------------------
'*
'*  Sends a character to a given port and returns with the port status.
'*
'*  Note: This function waits for room in the TX buffer, and if none
'*        ever becomes available (due to flow control) the system
'*        may hang.  Also see FossilPutch().
'*
      FUNCTION FossilPutchWait% (Port%, Char$)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H100 + ASC(Char$)     ' FOSSIL Service 01h

      CALL QBX00(InRegs, OutRegs)

      FossilPutchWait% = OutRegs.ax

      END FUNCTION

'*  FossilReadCursor()
'*---------------------------------------------------------------------------
'*
'*  Gets the current cursor position. (0,0) is the upper left corner.
'*
      FUNCTION FossilReadCursor% (Row%, Column%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.ax = &H1200                 ' FOSSIL Service 12h

      CALL QBX00(InRegs, OutRegs)

      Row% = OutRegs.dx / 256
      Column% = OutRegs.dx - Row% * 256

      FossilReadCursor% = 0

      END FUNCTION

'*  FossilReboot()
'*---------------------------------------------------------------------------
'*
'*  Reboots the system using a cold (Ctrl = 0) or warm (Ctrl = 1) boot.
'*
      FUNCTION FossilReboot% (Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.ax = &H1700 + Ctrl%         ' FOSSIL Service 17h

      CALL QBX00(InRegs, OutRegs)

      FossilReboot% = 0

      END FUNCTION

'*  FossilSetBaud()
'*---------------------------------------------------------------------------
'*
'*  Sets the FOSSIL baud rate for a given port (with N,8,1)
'*  and returns the port status.
'*
'*  Supported baud rates are:
'*     300, 600, 1200, 2400, 4800, 9600
'*     19200, 38400.  300 baud is assumed.
'*
      FUNCTION FossilSetBaud% (Port%, BaudRate&)

      DIM InRegs AS RegType, OutRegs AS RegType

      '*
      '* Convert the passed baud rate into a FOSSIL value and set the
      '* low-order bits for N,8,1.  Set AH = 0 for FOSSIL Service 00h.
      '*
      SELECT CASE BaudRate&
	 CASE 600
	    InRegs.ax = &H63
	 CASE 1200
	    InRegs.ax = &H83
	 CASE 2400
	    InRegs.ax = &HA3
	 CASE 4800
	    InRegs.ax = &HC3
	 CASE 9600
	    InRegs.ax = &HE3
	 CASE 19200
	    InRegs.ax = &H3
	 CASE 38400
	    InRegs.ax = &H23
	 CASE ELSE                        ' Assume 300 baud
	    InRegs.ax = &H43
      END SELECT

      InRegs.dx = Port%
      InRegs.flags = 512

      CALL QBX00(InRegs, OutRegs)

      FossilSetBaud% = OutRegs.ax

      END FUNCTION

'*  FossilStatus()
'*---------------------------------------------------------------------------
'*
'*  Returns the FOSSIL status for a given port.
'*
      FUNCTION FossilStatus% (Port%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H300                  ' FOSSIL Service 3h

      CALL QBX00(InRegs, OutRegs)

      FossilStatus% = OutRegs.ax

      END FUNCTION

'*  FossilWatchdog()
'*---------------------------------------------------------------------------
'*
'*  Enables (Ctrl = 1) or disables (Ctrl = 0) Watchdog for a given port.
'*
      FUNCTION FossilWatchdog% (Port%, Ctrl%)

      DIM InRegs AS RegType, OutRegs AS RegType

      InRegs.dx = Port%
      InRegs.ax = &H1400 + Ctrl%         ' FOSSIL Service 14h

      CALL QBX00(InRegs, OutRegs)

      FossilWatchdog% = 0

      END FUNCTION

